"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var React = require("react");
var pure_1 = require("../../utils/pure");
var Control_1 = require("../Control/Control");
var classnames = require("classnames");
var withTheme_1 = require("../../utils/withTheme");
var SteppableInput_1 = require("../SteppableInput/SteppableInput");
var with_defaults_1 = require("../../utils/with-defaults");
exports.TIME_INPUT = Symbol('TimeInput');
var ActiveSection;
(function (ActiveSection) {
    ActiveSection[ActiveSection["Hours"] = 0] = "Hours";
    ActiveSection[ActiveSection["Minutes"] = 1] = "Minutes";
})(ActiveSection = exports.ActiveSection || (exports.ActiveSection = {}));
var RawTimeInput = /** @class */ (function (_super) {
    tslib_1.__extends(RawTimeInput, _super);
    function RawTimeInput() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.state = {};
        _this.secondInput = false;
        _this.onHoursMouseDown = function (e) {
            if (!_this.props.isDisabled) {
                _this.setState({
                    activeSection: ActiveSection.Hours,
                });
                _this.correctMinutes();
            }
        };
        _this.onMinutesMouseDown = function (e) {
            if (!_this.props.isDisabled) {
                _this.setState({
                    activeSection: ActiveSection.Minutes,
                });
            }
        };
        _this.onIncrement = function () {
            _this.secondInput = false;
            _this.step(1);
        };
        _this.onDecrement = function () {
            _this.secondInput = false;
            _this.step(-1);
        };
        _this.onClear = function () {
            _this.secondInput = false;
            _this.updateStateTime();
        };
        _this.onFocus = function (e) {
            _this.secondInput = false;
            if (!isDefined(_this.state.activeSection)) {
                _this.setState({
                    activeSection: ActiveSection.Hours,
                });
            }
        };
        _this.onBlur = function (e) {
            _this.secondInput = false;
            _this.correctMinutes();
            _this.setState({
                activeSection: undefined,
            });
        };
        _this.onKeyDown = function (e) {
            var _a = _this.state, activeSection = _a.activeSection, hours = _a.hours, minutes = _a.minutes;
            switch (e.keyCode) {
                case Control_1.KeyCode.Left: {
                    e.preventDefault(); //block h-scrolling
                    if (activeSection === ActiveSection.Minutes) {
                        _this.secondInput = false;
                        _this.correctMinutes();
                        _this.setState({
                            activeSection: ActiveSection.Hours,
                        });
                    }
                    break;
                }
                case Control_1.KeyCode.Right: {
                    e.preventDefault(); //block h-scrolling
                    if (activeSection === ActiveSection.Hours) {
                        _this.secondInput = false;
                        _this.correctMinutes();
                        _this.setState({
                            activeSection: ActiveSection.Minutes,
                        });
                    }
                    break;
                }
                case Control_1.KeyCode.Delete: //fallthrough
                case Control_1.KeyCode.Backspace: {
                    _this.secondInput = false;
                    switch (activeSection) {
                        case ActiveSection.Minutes: {
                            _this.updateStateTime(hours, undefined);
                            break;
                        }
                        case ActiveSection.Hours: {
                            _this.updateStateTime(undefined, minutes);
                            break;
                        }
                    }
                    break;
                }
                default: {
                    var number = Control_1.KEY_CODE_NUM_MAP[e.keyCode];
                    if (isDefined(number)) {
                        _this.handleDigitKeyDown(number);
                    }
                }
            }
        };
        return _this;
    }
    RawTimeInput.prototype.componentWillMount = function () {
        var value = this.props.value;
        if (value) {
            var hours = value.hours, minutes = value.minutes;
            this.setState({
                hours: hours,
                minutes: minutes,
            });
        }
    };
    RawTimeInput.prototype.componentWillReceiveProps = function (newProps) {
        if (this.props.value !== newProps.value && isDefined(newProps.value)) {
            //value can be null here
            var hours = void 0;
            var minutes = void 0;
            if (newProps.value) {
                hours = newProps.value.hours;
                minutes = newProps.value.minutes;
            }
            this.setState({
                hours: hours,
                minutes: minutes,
            });
        }
    };
    RawTimeInput.prototype.render = function () {
        var _a, _b, _c;
        var _d = this.props, theme = _d.theme, decrementIcon = _d.decrementIcon, incrementIcon = _d.incrementIcon, isDisabled = _d.isDisabled, clearIcon = _d.clearIcon, error = _d.error, value = _d.value, SteppableInput = _d.SteppableInput;
        var _e = this.state, hours = _e.hours, minutes = _e.minutes, activeSection = _e.activeSection;
        var hoursClassName = classnames(theme.section, (_a = {},
            _a[theme.section_isActive] = !isDisabled && activeSection === ActiveSection.Hours,
            _a));
        var minutesClassName = classnames(theme.section, (_b = {},
            _b[theme.section_isActive] = !isDisabled && activeSection === ActiveSection.Minutes,
            _b));
        var onClear;
        if ((isDefined(value) && value !== null) || isDefined(hours) || isDefined(minutes)) {
            onClear = this.onClear;
        }
        var innerClassName = classnames(theme.inner, (_c = {},
            _c[theme.inner_isFilled] = Boolean(value),
            _c));
        return (React.createElement(SteppableInput, { isDisabled: isDisabled, theme: theme.SteppableInput, onBlur: this.onBlur, error: error, onFocus: this.onFocus, decrementIcon: decrementIcon, incrementIcon: incrementIcon, clearIcon: clearIcon, onKeyDown: this.onKeyDown, onClear: onClear, onDecrement: this.onDecrement, onIncrement: this.onIncrement },
            React.createElement("div", { className: innerClassName },
                React.createElement("span", { className: hoursClassName, onMouseDown: this.onHoursMouseDown }, this.format(hours)),
                React.createElement("span", { className: theme.separator }, ":"),
                React.createElement("span", { className: minutesClassName, onMouseDown: this.onMinutesMouseDown }, this.format(minutes)))));
    };
    RawTimeInput.prototype.format = function (value) {
        if (typeof value !== 'undefined') {
            return "" + (value >= 0 && value < 10 ? 0 : '') + value;
        }
        else {
            return '--';
        }
    };
    RawTimeInput.prototype.handleDigitKeyDown = function (digit) {
        var _a = this.state, hours = _a.hours, minutes = _a.minutes;
        switch (this.state.activeSection) {
            case ActiveSection.Hours: {
                if (this.secondInput && typeof hours !== 'undefined') {
                    var newHours = void 0;
                    if (hours < 2) {
                        newHours = Number("" + hours + digit);
                    }
                    else if (hours === 2) {
                        newHours = Math.min(Number("" + hours + digit), 23);
                    }
                    else {
                        newHours = digit;
                    }
                    this.updateStateTime(newHours, minutes);
                    this.setState({
                        activeSection: ActiveSection.Minutes,
                    });
                    this.secondInput = false;
                }
                else {
                    this.updateStateTime(digit, minutes);
                    if (digit > 2) {
                        this.setState({
                            activeSection: ActiveSection.Minutes,
                        });
                        this.secondInput = false;
                    }
                    else {
                        this.secondInput = true;
                    }
                }
                break;
            }
            case ActiveSection.Minutes: {
                if (this.secondInput && typeof minutes !== 'undefined') {
                    var newMinutes = Number("" + (minutes >= 10 ? ('' + minutes)[1] : minutes) + digit);
                    this.updateStateTime(hours, newMinutes);
                }
                else {
                    this.updateStateTime(hours, digit);
                    this.secondInput = true;
                }
            }
        }
    };
    RawTimeInput.prototype.step = function (amount) {
        var _a = this.state, hours = _a.hours, minutes = _a.minutes, activeSection = _a.activeSection;
        switch (activeSection) {
            case ActiveSection.Hours: {
                this.updateStateTime(add(hours, amount, 23), minutes);
                break;
            }
            case ActiveSection.Minutes: {
                this.updateStateTime(hours, add(Math.min(typeof minutes !== 'undefined' ? minutes : Infinity, 59), amount, 59));
                break;
            }
        }
    };
    RawTimeInput.prototype.updateStateTime = function (hours, minutes) {
        var _a = this.props, onValueChange = _a.onValueChange, value = _a.value;
        var canBuildValue = typeof hours !== 'undefined' && typeof minutes !== 'undefined' && minutes < 60;
        var newValueDiffers = canBuildValue &&
            (typeof value === 'undefined' || value === null || value.hours !== hours || value.minutes !== minutes);
        if (canBuildValue) {
            if (newValueDiffers) {
                onValueChange &&
                    onValueChange({
                        hours: hours,
                        minutes: minutes,
                    });
            }
        }
        else {
            if (isDefined(this.props.value)) {
                onValueChange && onValueChange(undefined);
            }
            this.setState({
                hours: hours,
                minutes: minutes,
            });
        }
    };
    RawTimeInput.prototype.correctMinutes = function () {
        if (typeof this.state.minutes !== 'undefined' && this.state.minutes >= 60) {
            this.updateStateTime(this.state.hours, 59);
        }
    };
    RawTimeInput = tslib_1.__decorate([
        pure_1.PURE
    ], RawTimeInput);
    return RawTimeInput;
}(React.Component));
exports.TimeInput = withTheme_1.withTheme(exports.TIME_INPUT)(with_defaults_1.withDefaults({
    SteppableInput: SteppableInput_1.SteppableInput,
})(RawTimeInput));
/**
 * Values can be zeros (start from 0). Max is included value.
 */
function add(a, b, max) {
    if (typeof a === 'undefined') {
        return b < 0 ? max : 0;
    }
    var result = (a + b) % (max + 1);
    if (result < 0) {
        result += max + 1;
    }
    return result;
}
function isDefined(value) {
    return typeof value !== 'undefined';
}
//# sourceMappingURL=TimeInput.js.map